/*
This code receives raw MIDI data in Hex fromat from the Frontier Designs Alphatrack
It then manages that data, reflecting it back to the unit so as to control the LED's, 
and reflect the position of the rotary knobs and the volume fader and the one and two finger ribbon slider.

It initialises the unit when the patch is openned and resets the unit, including clearing the LCD when the patch is closed

it has an OSC output so that the control data and the switch/button state can be used in other software packages

by garth paine	November 2007
http://www.activatedspace.com  and http://www.syncsonics.com
*/

// global code
inlets = 1;
outlets = 1;
autowatch = 1;

var midiMessage = new Array(0, 0, 0), hexNum = new Array(49, 50, 51), knobState = new Array(0, 0, 0, 0, 0, 0);
var inNum=0, i = 0, X = 0, y = 0, z = 0, knob = 0, a = 0, oneFinger = 0, twoFingers = 0, pNum = 0;
var buttonArray = new Array(130), buttonName = new Array(130);
var displayAddress = "240 0 1 64 32 0 ", bottomLine = "16 ", topLine = "0 ";
var volume = "", a = "",  volChar = "", printVolume = "";

function setup(){
	outlet(0, ("144 78 127")); //turn on green READ LED
	outlet(0, ("144 75 0")); //turn off RED  LED
	outlet(0, ("224 0 0"));  // set fader to zero
	knobState = [0, 0, 0, 0, 0, 0];
	midiMessage = [0, 0, 0];
	clearTopLCD();
	clearBottomLCD();
	outlet(0, (displayAddress + topLine + "72 101 108 108 111 32 71 97 114 116 104 247")); // print Hello Garth to LCD
	post("\n \n ---+----+----+----+---\nAlphatrack setup by \n Garth Paine 2007 \n http://www.activatedspace.com \n ---+----+----+----+---\n");
	buttonName[70] = "Shift";			//setup name tags so that OSC messages represent the button label
	buttonName[16] = "Mute";
	buttonName[8] = "Solo";
	buttonName[0] = "RecArm";
	buttonName[42] = "Pan";
	buttonName[41] = "Send";
	buttonName[44] = "eq";
	buttonName[43] = "Plugin";
	buttonName[74] = "Auto";
	buttonName[54] = "F1";
	buttonName[55] = "F2";
	buttonName[56] = "F3";
	buttonName[57] = "F4";
	buttonName[87] = "SkipBk";
	buttonName[88] = "SkipFwd";
	buttonName[86] = "Loop";
	buttonName[50] = "Flip";
	buttonName[91] = "rwd";
	buttonName[92] = "ffd";
	buttonName[93] = "Stop";
	buttonName[94] = "Play";
	buttonName[95] = "Record";
	buttonName[104] = "FaderTouch";
	buttonName[120] = "Knob0Touch";
	buttonName[121] = "Knob1Touch";
	buttonName[122] = "Knob2Touch";
	buttonName[32] = "Knob0Push";
	buttonName[33] = "Knob1Push";
	buttonName[34] = "Knob2Push";
	buttonName[116] = "TouchStrip";
	}

function closeDown(){
//turn off green READ LED
	for (x=0; x<100; x++){
		outlet(0, ("144 " + x + " 0"));}
		
		outlet(0, ("224 0 0"));
		outlet(0, (displayAddress + topLine + "32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 247"));  // clear topLine of LCD
		outlet(0, (displayAddress + bottomLine + "32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 247"));} // clear bottomLine of LCD


function list()
{
	var a = arrayfromargs(arguments);
	post("---------------" + "\n" +a +"\n");
}

function msg_int(inNum)
{
	if(midiMessage[i] != inNum){
		midiMessage[i] = inNum;}
	
	//MIDI NOTE Message
		if (i == 2 & midiMessage[0] == 144 ){
			//post("\n midiMessage " + midiMessage);
		//---------------------------------------------------------------------------------------------------------------
	// slider - one and two finger movement
		if (i == 2 & midiMessage[1] == 116){
			oneFinger = midiMessage[2]; }
				
		if (i == 2 & midiMessage[1] == 123){
			twoFingers = midiMessage[2];}

		else{
			doButton(midiMessage);}}
			
		if (oneFinger > 0){
			if ((i == 2) & (midiMessage[0] == 233)){
			writeToLCD(midiMessage[2], 25);
			messnamed (("oneFinger"), midiMessage[2]);
			messnamed(("OSCout"), ("oneFinger " + midiMessage[2]));}}	
			
		if (twoFingers > 0){
			if ((i == 2) & (midiMessage[0] == 233)){
			writeToLCD(midiMessage[2], 29);
			messnamed(("twoFingers"), midiMessage[2]);
			messnamed(("OSCout"), ("twoFingers " + midiMessage[2]));}}	
	
//-------------------------------------------------------------------------------------------------------------
	//Continuous Knobs Message
		
		if (i == 2 & midiMessage[0] == 176 ){										// If the MIDI message is a Continuous Knob
			knob = (midiMessage[1]-16);										// Get the current Knob value
		if((knobState[0] + knobState[1] + knobState[2]) <= 0)
			outlet(0, (displayAddress + topLine + "32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 247"));  // clear topLine of LCD
		if(midiMessage[2] == 1){												// If received Continuous Knob value is 1
			if (knobState[knob] <127)											// and If Continuous Knob value is < 127
				knobState[knob] = knobState[knob] + 1;}							// increment Continuous Knob value 
			else{
				if (knobState[knob] >0)										//else if Continuous Knob value > 0 decrement value
					knobState[knob] = knobState[knob] - 1;}
		
		writeToLCD(knobState[knob], (knob*5));								// Print the 3 Continuous Knob values to the  LCD  Screen	
		
		//post((("knob" + knob) + " " +  knobState[knob]));
		messnamed (("knob" + knob), knobState[knob]);
		messnamed(("OSCout"), (("Knob" + knob + " ") + knobState[knob]));}						//Send out knob data to receive objects knob0, knob1, knob2						

//---------------------------------------------------------------------------------------------------------------
	//PITCH BEND Message (also turn on/off RED LED when fader is touched)
		if (i == 2 & midiMessage[0] == 224 ){
			outlet(0, ("144 75 127")) 			// Turn on RED Write LED
				if (midiMessage[2] != pNum){
					outlet(0, "224 112 " + midiMessage[2]);
					messnamed (("fader"), midiMessage[2]);
					messnamed(("OSCout"), ("Fader " + midiMessage[2] ));
					writeToLCD(midiMessage[2], 20);
					pNum =  midiMessage[2];}}
		else if (i == 2 & midiMessage[1] == 104){
			outlet(0, ("144 75 0"));}
												// Turn off RED Write LED
	
// increment the counter to make sure all MIDI incoming messages are 3 items long
	i ++;
	if (i >2){
		i = 0;}
}

// function to operate Button States
function doButton(midiMessage){
		if(midiMessage[2] == 127){
			if (buttonArray[midiMessage[1]] == 127){
					buttonOFF(midiMessage);									// toggle the button LED
					buttonArray[midiMessage[1]] = 0;}
				else {
					buttonOn(midiMessage);	
					buttonArray[midiMessage[1]] = 127;}
					}}

// function to turn on LED associated with button
function buttonOn(midiMessage){
	if (midiMessage[1] == 104){
		//post("Touched Fader \n");  													// used to filter out fader movements from other functions
		//outlet(0, (midiMessage[0] + " 75" +" 127"))
		}
	else
	outlet(0, (midiMessage[0] + " " + midiMessage[1] +" 127"));							//toggle on LED for pushed button
	messnamed(("OSCout"), (buttonName[midiMessage[1]] +" 127"));
	} 								
	
	
// function to turn off LED associated with button
function buttonOFF(midiMessage){
	if (midiMessage[1] == 104){
		//outlet(0, (midiMessage[0] + " 75" +" 0"))
		}
	else
	outlet(0, (midiMessage[0] + " " + midiMessage[1] +" 0"));							//toggle off LED for pushed button
	messnamed(("OSCout"), (buttonName[midiMessage[1]] + " 0"));
	} 								
	
// function to write the fader volume and the continuous knob values to the LCD display
function writeToLCD(num, position){
			volume = (num).toString();  												//turn fader value into string to allow for Hex convertion for writing to screen
			
			for(y = 0; y < volume.length; y++){
				a = (parseInt(volume.charAt(y)) + 48).toString();  							//convert to Hex value to send to screen
				printVolume = printVolume + "  " + a;} 									//concatenate the values

			outlet(0, (displayAddress + bottomLine + "86 111 108 58 247")); 		// write Volume - to the screen
			outlet(0, (displayAddress + " " + position + printVolume + " 32  247"));    			// write fader value to the screen
			printVolume = ""; }  														//reset the displayed string to null so it can collect the next set of values from the fader
		
function clearTopLCD() {
	outlet(0, (displayAddress + topLine + "32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 247"));}  // clear topLine of LCD

function clearBottomLCD() {
	outlet(0, (displayAddress + bottomLine + "32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 32 247"));} // clear bottomLine of LCD

//The End  :-)